#include "SimplePOP3.h"
#include <avr/pgmspace.h>

SimplePOP3::SimplePOP3(void)
{
	sock = -1;
};

int SimplePOP3::begin( String h, String u, String p)
{
	return begin(h, u, p, POP3_PORT);
}

int SimplePOP3::begin( String h, String u, String p, int port )
{
	h.toCharArray(hostname, BUFSIZE);
	u.toCharArray(username, BUFSIZE);
	p.toCharArray(password, BUFSIZE);
	
	host = NULL;
	srv = NULL;
	
	host = gethostbyname(hostname);
	if( host == NULL ) {
		trace_error("gethostbyname %s", __func__);
		return -1;
	}
	
	memset( &sa, 0, sizeof(sockaddr_in) );
	memcpy( &sa.sin_addr, host->h_addr, sizeof(struct in_addr) );
	sa.sin_family = AF_INET;
	sa.sin_port = htons(port);
	sock = socket(AF_INET, SOCK_STREAM, 0);
	if( sock < 0 ) {
		trace_error("socket %s", __func__);
		return sock;
	}
	
	if( connect( sock, (struct sockaddr *)&sa, sizeof(sockaddr)) < 0) {
		trace_error("connect %s", __func__);
		sock = -1;
		return -1;
  	}
	
	// ڑbZ[Wǂ݂
	(void)receive();

	// OC
	String cmd = String("USER ");
	cmd.concat(username);
	String ret = sendCommand(cmd);
	if( ret == NULL ) {
    	close(sock);
    	sock = -1;
    	return -1;
 	}
	
	cmd = String("PASS ");
	cmd.concat(password);
	ret = sendCommand(cmd);
	if( ret == NULL ) {
		trace_error( "authenticate failed:%s", __func__);
		close(sock);
		sock = -1;
		return -1;
	}
	// ڑ  
	return 1;
}


// POP3T[o[ɑM
int SimplePOP3::transmit(char *msg)
{
	int length;
	if( sock < 0 ) return -1;
	length = strlen(msg);
	trace_debug("%s: %s, length=%d", __func__, msg , length);
	return send(sock, msg, length, 0);
}

// POP3T[o[M
char *SimplePOP3::receive(void)
{
	if( sock < 0 ) return NULL;
	memset(r_buf,0,BUFSIZE);
	if( recv(sock, r_buf, BUFSIZE, 0) < 0 )
		return NULL;
	trace_debug( "%s: %s", __func__, r_buf );
	return r_buf;
}

// POP3̃R}hs
String SimplePOP3::sendCommand( String cmd )
{
	if( sock < 0 ) return NULL;
	
	cmd.concat("\n");
	memset(s_buf,0,BUFSIZE);
	cmd.toCharArray(s_buf, BUFSIZE);
	if( transmit(s_buf) < 0 ){
		trace_error( "transmit error: %s" ,__func__);
		return NULL;
	}
	
	String rstr;
	do{
		// sǂݔ΂
		char *r = receive();
		if( r == NULL ) {
			trace_error( "receive error: %s", __func__);
			return NULL;
		}
		rstr = String(r);
		rstr.trim();
	}while( rstr.length() < 1 );
	
	String tmp = rstr.substring(0,3);
	if( tmp.compareTo("+OK") != 0 ) {
		rstr.toCharArray(r_buf, BUFSIZE );
		trace_error( "POP3: %s", r_buf );
		return NULL;
	}
	return rstr;
}

// POP3statR}hsǃ[Ԃ
int SimplePOP3::stat()
{
	if( sock < 0 ) return -1;
	
	String ret = sendCommand(String("stat"));

	if( ret != NULL ) {
		// c[Ԃ
		int to = ret.lastIndexOf(' ');
		String s_mails = ret.substring(3, to );
		s_mails.trim();
		Serial.println(s_mails);
   
		return s_mails.toInt();
	}
	return -1;
}

void SimplePOP3::end(void)
{
	if( sock < 0 ) return;
	sendCommand(String("QUIT"));
	close(sock);
	sock = -1;
}

SimplePOP3::~SimplePOP3(void)
{
	end();
}
